==> 0.0.2.tar.gz: lib/src/network/error_interceptor.dart <==
import 'package:dio/dio.dart';

class ErrorInterceptor extends QueuedInterceptor {
  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    switch (err.type) {
      case DioErrorType.cancel:
        return handler.next(RequestCancelledError(err.requestOptions));
      case DioErrorType.connectTimeout:
        return handler.next(RequestTimeoutError(err.requestOptions));
      case DioErrorType.sendTimeout:
        return handler.next(SendTimeoutError(err.requestOptions));
      case DioErrorType.receiveTimeout:
        return handler.next(ReceiveTimeoutError(err.requestOptions));
      case DioErrorType.response:
        switch (err.response?.statusCode) {
          case 400:
            return handler.next(BadRequestError(err.requestOptions));
          case 401:
          case 403:
            return handler.next(UnauthorizedError(err.requestOptions));
          case 404:
            return handler.next(NotFoundError(err.requestOptions));
          case 408:
            return handler.next(RequestTimeoutError(err.requestOptions));
          case 409:
            return handler.next(ConflictError(err.requestOptions));
          case 500:
            return handler.next(InternalServerErrorError(err.requestOptions));
          case 503:
            return handler.next(ServiceUnavailableError(err.requestOptions));
          default:
            return handler.next(UnexpectedError(err.requestOptions));
        }
      case DioErrorType.other:
        return handler.next(err);
    }
  }

  @override
  Future<void> onResponse(
    Response<dynamic> response,
    ResponseInterceptorHandler handler,
  ) async {
    // WarningError
    if (response.statusCode == 200 && response.data is Map<String, dynamic>) {
      final responseBody = response.data as Map<String, dynamic>;
      if (responseBody['error'] != null) {
        final error = responseBody['error'] as String;
        return handler.reject(WarningError(response.requestOptions, error));
      }
    }
    return handler.next(response);
  }
}

/// DioErrorType.cancel
class RequestCancelledError extends DioError {
  RequestCancelledError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Request cancelled';
  }
}

/// DioErrorType.connectTimeout || (DioErrorType.response && statusCode == 404)
class RequestTimeoutError extends DioError {
  RequestTimeoutError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Invalid request';
  }
}

/// DioErrorType.sendTimeout
class SendTimeoutError extends DioError {
  SendTimeoutError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Send timeout in connection with API server, please try again';
  }
}

/// DioErrorType.receiveTimeout
class ReceiveTimeoutError extends DioError {
  ReceiveTimeoutError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Receive timeout in connection with API server, please try again';
  }
}

/// DioErrorType.response && statusCode == 400
class BadRequestError extends DioError {
  BadRequestError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Invalid request';
  }
}

/// DioErrorType.response && (statusCode == 401 || statusCode == 403)
class UnauthorizedError extends DioError {
  UnauthorizedError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Access denied';
  }
}

/// DioErrorType.response && statusCode == 404
class NotFoundError extends DioError {
  NotFoundError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'The requested information could not be found';
  }
}

/// DioErrorType.response && statusCode == 409
class ConflictError extends DioError {
  ConflictError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Conflict occurred';
  }
}

/// DioErrorType.response && statusCode == 500
class InternalServerErrorError extends DioError {
  InternalServerErrorError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Unknown error occurred, please try again later';
  }
}

/// DioErrorType.response && statusCode == 503
class ServiceUnavailableError extends DioError {
  ServiceUnavailableError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Service unavailable';
  }
}

/// The others
class UnexpectedError extends DioError {
  UnexpectedError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Unexpected error';
  }
}

/// statusCode == 200, but has server defined error
class WarningError extends DioError {
  WarningError(RequestOptions r, this.errorCode) : super(requestOptions: r);

  final String errorCode;

  @override
  String toString() {
    return errorCode;
  }
}

==> 0.0.3-dev2.tar.gz: lib/src/network/error_interceptor.dart <==
import 'dart:convert';

import 'package:dio/dio.dart';

class ErrorInterceptor extends QueuedInterceptor {
  @override
  void onError(DioError err, ErrorInterceptorHandler handler) {
    switch (err.type) {
      case DioErrorType.cancel:
        return handler.next(RequestCancelledError(err.requestOptions));
      case DioErrorType.connectTimeout:
        return handler.next(RequestTimeoutError(err.requestOptions));
      case DioErrorType.sendTimeout:
        return handler.next(SendTimeoutError(err.requestOptions));
      case DioErrorType.receiveTimeout:
        return handler.next(ReceiveTimeoutError(err.requestOptions));
      case DioErrorType.response:
        switch (err.response?.statusCode) {
          case 400:
            return handler.next(BadRequestError(err.requestOptions));
          case 401:
          case 403:
            return handler.next(UnauthorizedError(err.requestOptions));
          case 404:
            return handler.next(NotFoundError(err.requestOptions));
          case 408:
            return handler.next(RequestTimeoutError(err.requestOptions));
          case 409:
            return handler.next(ConflictError(err.requestOptions));
          case 500:
            return handler.next(InternalServerErrorError(err.requestOptions));
          case 503:
            return handler.next(ServiceUnavailableError(err.requestOptions));
          default:
            return handler.next(UnexpectedError(err.requestOptions));
        }
      case DioErrorType.other:
        return handler.next(err);
    }
  }

  @override
  Future<void> onResponse(
    Response<dynamic> response,
    ResponseInterceptorHandler handler,
  ) async {
    // WarningError
    if (response.statusCode == 200 && response.data is Map<String, dynamic>) {
      final responseBody = response.data as Map<String, dynamic>;
      if (responseBody['error'] != null) {
        final errorCode = responseBody['error'] as int;
        final subCode = responseBody['subCode'] as int;
        final errorMessage = responseBody['message'] as String;
        final detail = responseBody['detail'];
        return handler.reject(
          WarningError(
            response.requestOptions,
            errorCode: errorCode,
            subCode: subCode,
            errorMessage: errorMessage,
            detail: detail,
          ),
        );
      }
    }
    return handler.next(response);
  }
}

/// DioErrorType.cancel
class RequestCancelledError extends DioError {
  RequestCancelledError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Request cancelled';
  }
}

/// DioErrorType.connectTimeout || (DioErrorType.response && statusCode == 404)
class RequestTimeoutError extends DioError {
  RequestTimeoutError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Invalid request';
  }
}

/// DioErrorType.sendTimeout
class SendTimeoutError extends DioError {
  SendTimeoutError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Send timeout in connection with API server, please try again';
  }
}

/// DioErrorType.receiveTimeout
class ReceiveTimeoutError extends DioError {
  ReceiveTimeoutError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Receive timeout in connection with API server, please try again';
  }
}

/// DioErrorType.response && statusCode == 400
class BadRequestError extends DioError {
  BadRequestError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Invalid request';
  }
}

/// DioErrorType.response && (statusCode == 401 || statusCode == 403)
class UnauthorizedError extends DioError {
  UnauthorizedError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Access denied';
  }
}

/// DioErrorType.response && statusCode == 404
class NotFoundError extends DioError {
  NotFoundError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'The requested information could not be found';
  }
}

/// DioErrorType.response && statusCode == 409
class ConflictError extends DioError {
  ConflictError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Conflict occurred';
  }
}

/// DioErrorType.response && statusCode == 500
class InternalServerErrorError extends DioError {
  InternalServerErrorError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Unknown error occurred, please try again later';
  }
}

/// DioErrorType.response && statusCode == 503
class ServiceUnavailableError extends DioError {
  ServiceUnavailableError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Service unavailable';
  }
}

/// The others
class UnexpectedError extends DioError {
  UnexpectedError(RequestOptions r) : super(requestOptions: r);

  @override
  String toString() {
    return 'Unexpected error';
  }
}

/// statusCode == 200, but has server defined error
class WarningError extends DioError {
  WarningError(
    RequestOptions r, {
    required this.errorCode,
    required this.subCode,
    required this.errorMessage,
    this.detail,
  }) : super(requestOptions: r);

  final int errorCode;
  final int subCode;
  final String errorMessage;
  final dynamic detail;

  @override
  String toString() {
    return jsonEncode(<String, dynamic>{
      'errorCode': errorCode,
      'subCode': subCode,
      'message': errorMessage,
    });
  }
}
